/*
	int_nmi.c
	2012 by Toni Castillo Girona
	<toni.castillo@fa.upc.edu>

	This trivial C-program issues a INT $0x2 interrupt, that is,
	it does simulate there is a NMI interrupt requesting OS attention.
	NMI stands for "Non Maskable Interrupts", and they are used to
	take care of hardware-related problems, like memory parity errors and
	the sort.

	Our main purpose is to provoke a call to do_IRQ() and thus making
	current to yield the processor (as long as p is running on the SAME
	processor taking care of the unknwon NMI interrupt).

	NOTE:	This interrupt cannot be issued from user-space unless the
	IDT has been previously altered so its DPL=3.

	USAGE:	./int_nmi -n NUMBER

	If NUMBER = -1, it loops forever sending NMI interrupts.

*/
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>

void _send_nmi (void);

int main (int argc, char **argv){

	int c, number, i;

	/* Get the argument from the cli: */
	c = getopt(argc, argv,"n:");
	if(c<0||(c>0 && c!='n')){
		printf("Usage: int_nmi -n number.\n");
		exit(-1);
	} 

	number = atoi(optarg);

	/* Send some NMI interrupt requests to the GNU/Linux Kernel */
	for(i=0;
		 (number>0)?i<number:i==i;
		_send_nmi(),	/* This is the actual NMI */
		((number==-1)?0:++i)
	);

	/* This is not going to be executed, this process is killed */
	return 0;

}

/*
	This function does send a INT 0x2 NMI interrupt
*/
void _send_nmi () {
	__asm__ __volatile__(
		"int $0x2\n"
   	 );
}
