#!/bin/bash
#
#	irq_divert.sh
#	2012 by Toni Castillo Girona
#	<toni.castillo@fa.upc.edu>
#
#	This script divert all IRQ lines according to processor_mask
#	NOTE: not all the IRQS can be easily diverted. In this case, 
#	the old affinity wouldn't change.
#
#	USAGE:	./irq_divert.sh <new_cpu_mask>
#
#
IRQS="/proc/irq"

# Mask from the CLI
if [ ! $# -eq 1 ]; then
	echo "Usage: `basename $0` processor_mask"
	exit 1
fi

curdir=`pwd`

cd $IRQS || exit 2
echo "Trying to set new affinity: $1"
for irq in *; do
	# Avoid "default...":
	if [ "$irq" != "default_smp_affinity" ]; then
		# Divert this irq:
		oaf=`cat $irq/smp_affinity`	# old affinity
		echo "$1" > $irq/smp_affinity 2>/dev/null
		echo -ne "IRQ line:[$irq]\t"	\
			"Old affinity:[$oaf]\t"		\
			"New affinity:[`cat $irq/smp_affinity`]\n"
	fi
done

cd $curdir
exit 0
